//========================================
//TCard by George Deng (daogra@sina.com)
//
//THE MOST EFFICENT CARD COMPONENT
//    EVER MADE FOR DELPHI !
//
//This component is a freeware designed
//    to facilitate card game programming.
//    If you're planning for a game like 
//    M$ Card, Congratulation! You got
//    the best choice availabe on internet!
//
//This is a freeware, which means you can 
//    use and disperse and modify it at will,
//    but I'll appreciate if you keep my name
//    intact in this section.
//
//(end of cheap self promoting)
//========================================
//A brief overview:
//    property CardID;      
//        Integer from 1 to 52, as Club A to Spade K goes.
//    property Value;
//        Integer form 1 to 13, represents Ace to K.
//    property Suit;
//        [S, C, D, H] as Spade, Club, Diamond, Hearts.
//    property ShowDeck;
//        Boolean, true means you got the Deck image of a card.
//    property DeckType;
//        [Standard, Beach, Castle] just alter the value in the property editor and see the results.
//    property SelectedCard;
//        Boolean, true means a card is selected, which is shown in reversed color.
//
//    procedure SetCardID(aCardID : TCardID);
//        aCardID is an integer in [1..52].
//    procedure SetCard(CValue: Integer; CSuit: TCardSuit);
//        CValue is integer in [1..13] while CSuit in [S, C, D, H].
//    procedure Turn;
//        Use Turn to change the Shown side of the card.
//========================================
unit Card;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs;

const
  D_WIDTH     = 71;
  D_HEIGHT    = 96;

type
  TCardID = 1..52;
  TCardValue = 1..13;
  TCardSuit = (S,D,C,H);
  TDecks = (Standard, Beach, Castle);
  TCard = class(TGraphicControl)
  private
    FCardID: TCardID;
    FValue: TCardValue;
    FSuit: TCardSuit;
    FDeckType: TDecks;
    FDeckID : byte;
    FShowDeck: Boolean;
    FSuitID : byte;
    FSelectedCard: Boolean;
    procedure WMSize(var Message: TWMSize); message WM_PAINT;
    procedure SetValue(Value : TCardValue);
    procedure SetSuit(Suit: TCardSuit);
    procedure SetShowDeck(ShowDeck: Boolean);
    procedure SetDeckType(DeckType: TDecks);
    procedure SetSelectedCard(CardSelectedStatus: Boolean);
  protected
    property Height default D_HEIGHT;
    property Width default D_WIDTH;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetCardID(aCardID : TCardID);
    procedure SetCard(CValue: Integer; CSuit: TCardSuit);
    procedure Turn;
  published
    property CardID: TCardID read FCardID write SetCardID;
    property Value: TCardValue read FValue write SetValue;
    property Suit: TCardSuit read FSuit write SetSuit;
    property ShowDeck: Boolean read FShowDeck write SetShowDeck;
    property DeckType: TDecks read FDeckType write SetDeckType;
    property SelectedCard: Boolean read FSelectedCard write SetSelectedCard;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

var
  CardInstanceCount: LongInt = 0;
  CardSet: TBitmap;
  CardBack: TBitmap;
  CardMask: TBitmap;
  CardWork: TBitmap;

procedure Register;

implementation

{$R CARD.RES}
//========================================
//
//========================================
constructor TCard.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := D_HEIGHT;
  Width := D_WIDTH;
  FCardID := 1;
  FValue := 1;
  FSuit := H;
  FDeckType := Standard;
  FShowDeck := False;
  Randomize;
  if (CardInstanceCount < 1) then
    begin
    CardSet := TBitmap.Create;
    CardMask := TBitmap.Create;
    CardMask.Monochrome := True;
    CardBack := TBitmap.Create;
    CardWork := TBitmap.Create;
    CardWork.Width := D_WIDTH;
    CardWork.Height := D_HEIGHT;
    CardSet.LoadFromResourceName(HInstance, 'CARDSET');
    CardMask.LoadFromResourceName(HInstance, 'CARDMASK');
    CardBack.LoadFromResourceName(HInstance, 'CARDBACK');
    end;
  Inc(CardInstanceCount);
end;
//========================================
//
//========================================
destructor TCard.Destroy;
begin
  Dec(CardInstanceCount);
  if (CardInstanceCount < 1) then
    begin
    CardSet.Free;
    CardMask.Free;
    CardBack.Free;
    CardWork.Free;
    end;
  inherited Destroy;
end;
//========================================
//
//========================================
procedure TCard.WMSize(var Message: TWMSize); // No resizing allowed !
begin
  inherited;
  Width := D_WIDTH;
  Height := D_HEIGHT;
end;
//========================================
//
//========================================
procedure TCard.Paint;
var
  CardPaintMode: Integer;
begin
  BitBlt(CardWork.Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
      Canvas.Handle, 0, 0, SRCCOPY);
  if (FSelectedCard = False) then
    begin
    CardPaintMode := SRCPAINT;
    BitBlt(CardWork.Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
    CardMask.Canvas.Handle, 0, 0, SRCAND);
    end
  else
    begin
    CardPaintMode := NOTSRCERASE;
    BitBlt(CardWork.Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
      CardMask.Canvas.Handle, 0, 0, SRCERASE);
    end;
  case FShowDeck of
    False:
      BitBlt(CardWork.Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
        CardSet.Canvas.Handle, (FValue-1) * D_WIDTH,
        FSuitID * D_HEIGHT, CardPaintMode);
    True:
      BitBlt(CardWork.Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
        CardBack.Canvas.Handle, 0, FDeckID * D_HEIGHT, CardPaintMode);
  end;
  BitBlt(Canvas.Handle, 0, 0, D_WIDTH, D_HEIGHT,
    CardWork.Canvas.Handle, 0, 0, SRCCOPY);
end;
//========================================
//
//========================================
procedure TCard.SetValue(Value: TCardValue);
begin
  if Value<1 then Value := 1;
  if Value>13 then Value :=13;
  FValue := Value;
  FCardID := 4*(FValue -1) + FSuitID+1;
  if not FShowDeck then Repaint;
end;
//========================================
//
//========================================
procedure TCard.SetSuit(Suit: TCardSuit);
begin
  FSuit := Suit;
  Case Suit of
    S : FSuitID := 3;
    H : FSuitID := 2;
    D : FSuitID := 1;
    C : FSuitID := 0;
  end;
  FCardID := 4*(FValue -1) + FSuitID+1;
  if not FShowDeck then Repaint;
end;
//========================================
//
//========================================
procedure TCard.SetShowDeck(ShowDeck: Boolean);
begin
  FShowDeck := ShowDeck;
  RePaint;
end;
//========================================
//
//========================================
procedure TCard.SetCard(CValue: Integer; CSuit: TCardSuit);
begin
  if (CValue>0) and (CValue<14) then Value := CValue;
  if CSuit in [C,H,S,D] then Suit := CSuit;
end;
//========================================
//
//========================================
procedure TCard.SetCardID(aCardID: TCardID);
begin
  Value := ((aCardID-1) div 4) +1;
  Case (aCardID mod 4) of
    0 : Suit := S;
    1 : Suit := C;
    2 : Suit := D;
    3 : Suit := H;
  end;
  FCardID := aCardID;
end;
//========================================
//
//========================================
procedure TCard.Turn;
begin
  ShowDeck := not ShowDeck;
end;
//========================================
//
//========================================
procedure TCard.SetDeckType(DeckType: TDecks);
begin
  FDeckType := DeckType;
  Case DeckType of
    Standard : FDeckID := 0;
    Beach    : FDeckID := 1;
    Castle   : FDeckID := 2;
  end;
  RePaint;
end;
//========================================
//
//========================================
procedure TCard.SetSelectedCard(CardSelectedStatus: Boolean);
begin
  if (FSelectedCard <> CardSelectedStatus) then
    begin
    FSelectedCard := CardSelectedStatus;
    Repaint;
    end;
end;
//========================================
//
//========================================
procedure Register;
begin
  // change here to set the palette page for the TCard component
  RegisterComponents('Samples', [TCard]);
end;
//========================================
//
//========================================
end.
